@@ -644,8 +644,8 @@ find_program_check_fallback(struct workspace *wk, struct find_program_ctx *ctx,
 		obj_array_flatten_one(wk, fallback_arr, &subproj_name);
 
 		obj subproj;
-		if (!subproject(wk, subproj_name, requirement_auto, ctx->default_options, NULL, &subproj)
-			&& get_obj_subproject(wk, subproj)->found) {
+		if (!(subproject(wk, subproj_name, requirement_auto, ctx->default_options, NULL, &subproj)
+			&& get_obj_subproject(wk, subproj)->found)) {
 			return true;
 		}
 
@@ -656,9 +656,10 @@ find_program_check_fallback(struct workspace *wk, struct find_program_ctx *ctx,
 			if (!obj_dict_index(wk, wk->find_program_overrides[ctx->machine], prog, &_)) {
 				vm_warning_at(wk,
 					0,
-					"subproject %o claims to provide %o, but did not override it",
+					"subproject %o claims to provide %o for the %s machine, but did not override it",
 					subproj_name,
-					prog);
+					prog,
+					machine_kind_to_s(ctx->machine));
 			}
 		}
 	}
@@ -1033,6 +1034,8 @@ func_assert(struct workspace *wk, obj _, obj *res)
 	if (!get_obj_bool(wk, an[0].val)) {
 		if (an[1].set) {
 			LOG_E("%s", get_cstr(wk, an[1].val));
+		} else {
+			vm_error(wk, "assertion failed");
 		}
 		return false;
 	}
@@ -2305,6 +2308,7 @@ const struct func_impl impl_tbl_kernel[] =
 
 const struct func_impl impl_tbl_kernel_internal[] = {
 	{ "assert", func_assert, .flags = func_impl_flag_throws_error },
+	{ "configure_file", func_configure_file, tc_file },
 	{ "configuration_data", func_configuration_data, tc_configuration_data },
 	{ "disabler", func_disabler, tc_disabler },
 	{ "environment", func_environment, tc_environment },
