@@ -421,6 +421,14 @@ get_dependency_extraframework(struct workspace *wk, struct dep_lookup_ctx *ctx,
 
 					s = e + 1;
 				}
+			} else {
+				run_cmd_print_error(&cmd_ctx, log_debug);
+
+				L("failed to determine compiler framework directories");
+
+				// Add a dummy framework directory so we can still utilize it
+				// when building the cache key.
+				obj_array_push(wk, comp->fwdirs, make_str(wk, "unknown://"));
 			}
 			run_cmd_ctx_destroy(&cmd_ctx);
 
@@ -455,17 +463,21 @@ get_dependency_extraframework(struct workspace *wk, struct dep_lookup_ctx *ctx,
 		obj dep = 0;
 
 		obj_array_for(wk, comp->fwdirs, fw_dir) {
-			struct get_dependency_extraframework_scan_path_ctx scan_path_ctx = {
-				.wk = wk,
-				.fw = get_str(wk, fw),
-			};
-			fs_dir_foreach(
-				get_str(wk, fw_dir)->s, &scan_path_ctx, get_dependency_extraframework_scan_path_cb);
-			if (!scan_path_ctx.res) {
-				continue;
-			}
+			const char *fw_dir_str = get_str(wk, fw_dir)->s;
+			if (fs_dir_exists(fw_dir_str)) {
+				struct get_dependency_extraframework_scan_path_ctx scan_path_ctx = {
+					.wk = wk,
+					.fw = get_str(wk, fw),
+				};
+				fs_dir_foreach(get_str(wk, fw_dir)->s,
+					&scan_path_ctx,
+					get_dependency_extraframework_scan_path_cb);
+				if (!scan_path_ctx.res) {
+					continue;
+				}
 
-			fw = scan_path_ctx.res;
+				fw = scan_path_ctx.res;
+			}
 
 			obj fw_path;
 			TSTR(fw_path_buf);
@@ -1446,7 +1458,7 @@ func_declare_dependency(struct workspace *wk, obj _, obj *res)
 		[kw_dependencies] = { "dependencies", TYPE_TAG_LISTIFY | tc_dependency },
 		[kw_version] = { "version", obj_string },
 		[kw_include_directories] = { "include_directories", TYPE_TAG_LISTIFY | tc_coercible_inc },
-		[kw_variables] = { "variables", tc_array | tc_dict },
+		[kw_variables] = { "variables", tc_string | tc_array | tc_dict },
 		[kw_compile_args] = { "compile_args", TYPE_TAG_LISTIFY | obj_string },
 		[kw_objects] = { "objects", TYPE_TAG_LISTIFY | tc_file | tc_string },
 		[kw_extra_files] = { "extra_files", TYPE_TAG_LISTIFY | tc_coercible_files }, // ignored
