@@ -44,6 +44,9 @@ parse_config_string(struct workspace *wk, const struct str *ss, struct option_ov
 	if (str_has_null(ss)) {
 		LOG_E("option cannot contain NUL");
 		return false;
+	} else if (!key_only && !memchr(ss->s, '=', ss->len)) {
+		LOG_E("option must contain =, got %s", ss->s);
+		return false;
 	}
 
 	struct str subproject = { 0 }, key = { 0 }, val = { 0 }, cur = { 0 };
@@ -94,9 +97,6 @@ parse_config_string(struct workspace *wk, const struct str *ss, struct option_ov
 	} else if (key_only && val.len) {
 		LOG_E("unexpected '=' in option '%s'", ss->s);
 		return false;
-	} else if (!key_only && !val.len) {
-		LOG_E("expected '=' in option '%s'", ss->s);
-		return false;
 	}
 
 	oo->name = make_strn(wk, key.s, key.len);
@@ -454,38 +454,35 @@ set_option(struct workspace *wk, obj opt, obj new_val, enum option_value_source
 {
 	struct obj_option *o = get_obj_option(wk, opt);
 
-	// Only set options that haven't set from a source with higher
-	// precedence.  This means that option precedence doesn't have to rely
-	// on the order in which options are set.  This means that e.g.
-	// command-line set options can be set before the main meson.build file
-	// has even been parsed.
-	//
-	// This mostly follows meson's behavior, except that deprecated options
-	// cannot override command line options.
+	// Only set options that have not been set from a source with higher
+	// precedence.
 
-	/* { */
-	/* 	const char *sourcenames[] = { */
-	/* 		[option_value_source_unset] = "unset", */
-	/* 		[option_value_source_default] = "default", */
-	/* 		[option_value_source_environment] = "environment", */
-	/* 		[option_value_source_default_options] = "default_options", */
-	/* 		[option_value_source_subproject_default_options] = "subproject_default_options", */
-	/* 		[option_value_source_yield] = "yield", */
-	/* 		[option_value_source_commandline] = "commandline", */
-	/* 		[option_value_source_deprecated_rename] = "deprecated rename", */
-	/* 		[option_value_source_override_options] = "override_options", */
-	/* 	}; */
-	/* 	obj_fprintf(wk, log_file(), */
-	/* 		"%s option %o to %o from %s, last set by %s\n", */
-	/* 		o->source > source */
-	/* 		? "\033[31mnot setting\033[0m" */
-	/* 		: "\033[32msetting\033[0m", */
-	/* 		o->name, new_val, sourcenames[source], sourcenames[o->source]); */
-	/* } */
-
-	if (o->source > source) {
+	if (o->source >= source) {
 		return true;
 	}
+
+	if (false) {
+		const char *sourcenames[] = {
+			[option_value_source_unset] = "unset",
+			[option_value_source_default] = "default",
+			[option_value_source_environment] = "environment",
+			[option_value_source_default_options] = "default_options",
+			[option_value_source_subproject_default_options] = "subproject_default_options",
+			[option_value_source_yield] = "yield",
+			[option_value_source_commandline] = "commandline",
+			[option_value_source_deprecated_rename] = "deprecated rename",
+			[option_value_source_override_options] = "override_options",
+		};
+		obj_lprintf(wk,
+			log_debug,
+			"%s option %o to %o from %s, last set by %s\n",
+			o->source > source ? "\033[31mnot setting\033[0m" : "\033[32msetting\033[0m",
+			o->name,
+			new_val,
+			sourcenames[source],
+			sourcenames[o->source]);
+	}
+
 	o->source = source;
 
 	if (get_obj_type(wk, o->deprecated) == obj_bool && get_obj_bool(wk, o->deprecated)) {
@@ -657,7 +654,7 @@ set_compile_opt_from_env(struct workspace *wk, const char *name, const char *fla
 }
 
 static void
-set_str_opt_from_env(struct workspace *wk, const char *env_name, const char *opt_name)
+set_str_opt_from_env(struct workspace *wk, const char *env_name, const char *opt_name, const char *split)
 {
 	obj opt;
 	if (!get_option(wk, NULL, &STRL(opt_name), &opt)) {
@@ -666,7 +663,14 @@ set_str_opt_from_env(struct workspace *wk, const char *env_name, const char *opt
 
 	const char *env_val;
 	if ((env_val = os_get_env(env_name)) && *env_val) {
-		set_option(wk, opt, make_str(wk, env_val), option_value_source_environment, false);
+		obj val;
+		if (split) {
+			val = str_split(wk, &STRL(env_val), &STRL(split));
+		} else {
+			val = make_str(wk, env_val);
+		}
+
+		set_option(wk, opt, val, option_value_source_environment, false);
 	}
 }
 
@@ -919,7 +923,7 @@ init_global_options(struct workspace *wk)
 		make_compiler_env_option(wk, langs[i].l, toolchain_component_linker);
 	}
 
-	set_str_opt_from_env(wk, "PKG_CONFIG_PATH", "pkg_config_path");
+	set_str_opt_from_env(wk, "PKG_CONFIG_PATH", "pkg_config_path", ENV_PATH_SEP_STR);
 
 	set_binary_from_env(wk, "AR", "env.AR");
 	set_binary_from_env(wk, "NINJA", "env.NINJA");
@@ -1422,7 +1426,7 @@ list_options_for_subproject(struct workspace *wk, struct subprojects_common_ctx
 	TSTR(cwd);
 
 	struct wrap wrap = { 0 };
-	if (!wrap_parse(wk, path, &wrap)) {
+	if (!wrap_parse(wk, get_str(wk, current_project(wk)->subprojects_dir)->s, path, &wrap)) {
 		return ir_err;
 	}
 
