@@ -35,6 +35,7 @@ static const char *wrap_field_names[wrap_fields_count] = {
 	[wf_patch_hash] = "patch_hash",
 	[wf_patch_directory] = "patch_directory",
 	[wf_diff_files] = "diff_files",
+	[wf_method] = "method",
 	[wf_source_url] = "source_url",
 	[wf_source_fallback_url] = "source_fallback_url",
 	[wf_source_filename] = "source_filename",
@@ -269,8 +270,8 @@ wrap_parse_provides_cb(void *_ctx,
 		error_messagef(src, location, log_error, "empty provides key \"%s\"", k);
 		return false;
 	} else if (!*v) {
-		error_messagef(src, location, log_error, "empty provides value \"%s\"", v);
-		return false;
+		error_messagef(src, location, log_warn, "empty provides value \"%s\"", v);
+		return true;
 	}
 
 	ctx->add_provides_tgt = 0;
@@ -478,10 +479,13 @@ validate_wrap(struct wrap_parse_ctx *ctx, const char *file)
 	uint32_t i;
 
 	enum req { invalid, required, optional };
-	enum req field_req[wrap_fields_count] = { [wf_directory] = optional,
+	enum req field_req[wrap_fields_count] = {
+		[wf_directory] = optional,
 		[wf_patch_directory] = optional,
 		[wf_diff_files] = optional,
-		[wf_wrapdb_version] = optional };
+		[wf_wrapdb_version] = optional,
+		[wf_method] = optional,
+	};
 
 	if (ctx->wrap.fields[wf_patch_url] || ctx->wrap.fields[wf_patch_filename] || ctx->wrap.fields[wf_patch_hash]) {
 		field_req[wf_patch_url] = optional;
@@ -551,10 +555,9 @@ wrap_destroy(struct wrap *wrap)
 }
 
 bool
-wrap_parse(struct workspace *wk, const char *wrap_file, struct wrap *wrap)
+wrap_parse(struct workspace *wk, const char *subprojects, const char *wrap_file, struct wrap *wrap)
 {
 	bool res = false;
-	TSTR(subprojects);
 	struct wrap_parse_ctx ctx = { 0 };
 
 	if (!ini_parse(wrap_file, &ctx.wrap.src, &ctx.wrap.buf, wrap_parse_cb, &ctx)) {
@@ -585,8 +588,7 @@ wrap_parse(struct workspace *wk, const char *wrap_file, struct wrap *wrap)
 		dir = wrap->name.buf;
 	}
 
-	path_dirname(wk, &subprojects, wrap_file);
-	path_join(wk, &wrap->dest_dir, subprojects.buf, dir);
+	path_join(wk, &wrap->dest_dir, subprojects, dir);
 
 	res = true;
 ret:
@@ -1168,7 +1170,7 @@ wrap_handle_async(struct workspace *wk, const char *wrap_file, struct wrap_handl
 		ctx->sub_state = wrap_handle_sub_state_running;
 		timer_start(&ctx->duration);
 
-		if (!wrap_parse(wk, wrap_file, &ctx->wrap)) {
+		if (!wrap_parse(wk, ctx->opts.subprojects, wrap_file, &ctx->wrap)) {
 			return false;
 		}
 
@@ -1377,7 +1379,7 @@ wrap_load_all_iter(void *_ctx, const char *file)
 	}
 
 	struct wrap wrap = { 0 };
-	if (!wrap_parse(ctx->wk, ctx->path->buf, &wrap)) {
+	if (!wrap_parse(ctx->wk, ctx->subprojects, ctx->path->buf, &wrap)) {
 		return ir_err;
 	}
 
